"""Enumeration of configuration file formats supported by the ``Configurator``."""

from enum import Enum


class ConfigType(Enum):
    """Supported configuration file formats for ``Configurator.init_transports``."""

    NOT_SUPPORTED = 0
    CFG_OR_INI = 1
    JSON = 2
    TOML = 3
    XML = 4
    YAML = 5
