"""Enumeration of supported transport types."""

from ..utilities.compatibility import StrEnum


class BusType(StrEnum):
    """
    Supported transport categories.

    Used as registry keys in ``Station.add_handler`` to register handlers
    for all lines of a given type (e.g. all Kafka lines).
    """

    IN_MEMORY = "in_memory"
    AZURE = "azure"
    AWS = "aws"
    RABBIT_MQ = "rabbitmq"
    KAFKA = "kafka"
