from ..utilities.compatibility import StrEnum


class LetterLevel(StrEnum):
    """
    An enumeration defining the different severity levels for log messages.

    Each member has a string value representing its name, which can be
    useful for string-based comparisons or display.
    """

    TRACE = "TRACE"
    DEBUG = "DEBUG"
    INFO = "INFO"
    WARN = "WARN"
    ERROR = "ERROR"
    FATAL = "FATAL"
