def is_not_builtin_type(my_var):
    """
    Checks if a variable is not a common built-in type
    (int, str, float, bool, list, tuple, dict, set, complex, type, object, NoneType).

    Args:
        my_var: The variable to check.

    Returns:
        True if the variable is not a built-in type, False otherwise.
    """
    builtin_types = (
        int,
        str,
        float,
        bool,
        list,
        tuple,
        dict,
        set,
        complex,
        type,
        type(None),
    )

    is_builtin_type = False

    for builtin_type in builtin_types:
        if isinstance(my_var, builtin_type):
            is_builtin_type = True

    return not is_builtin_type
